
/*****************************************************************************/
/*                                                                           */
/*    COPYRIGHT : 5765 - 196                                                 */
/*                5765 - 476                                                 */
/*                5765 - 477                                                 */
/*                                                                           */
/*    @(#)(C) Copyright IBM Corp. 1993, 1995, 1996                           */
/*                                                                           */
/*    ALL RIGHTS RESERVED                                                    */
/*    LICENSED MATERIALS - PROPERTY OF IBM                                   */
/*                                                                           */
/* Product:         NetView Distribution Manager/6000                        */
/*                                                                           */
/* Module:          fndcx.c                                                  */
/*                                                                           */
/* Component:       Server and Daca User-Exit Routines                       */
/*                                                                           */
/* Description:     This module contains the SERVER and DACA User-Exit       */
/*                  and Notification routines.                               */
/*                  The following routines are provided.                     */
/*                                                                           */
/*                  - cx_daca_filename  modify local filename                */
/*                  - cx_daca_report    when request received                */
/*                  - cx_daca_request   before report sent                   */
/*                  - sx_server_request before request queued to DACA        */
/*                  - sx_server_report  after report processing              */
/*                                                                           */
/* Prerequisites:   fndhdr.h                                                 */
/*                  fndshrr.h                                                */
/*                  fndcx.h                                                  */
/*                                                                           */
/*****************************************************************************/

/*****************************************************************************/
/*                                                                           */
/* If the DVA_TEST_USER_EXITS preprocessor macro is defined, the sample      */
/* version of the user exits is compiled.                                    */
/* The sample user exits look for an environment variable called             */
/* "fnduex_trc". If this environment variable has been defined, it is        */
/* considered as the full path name of a trace file.                         */
/* This file is appended by the user exits with some trace data.             */
/*                                                                           */
/*****************************************************************************/
/*
define DVA_TEST_USER_EXITS
*/

#include <fndhdr.h>
#include <fndshrr.h>
#include <fndcx.h>

#ifdef DVA_TEST_USER_EXITS

#include <stdio.h>
#define FNDUEX_TRC "fnduex_trc"
DC_VOID  dump_dtime(FILE *trcfile, SCHED_TIME *dtime, DC_CHAR *text);
DC_VOID  glob_to_dot(DC_CHAR *dname, DC_GLOB *gname);
DC_VOID  print_address_list(FILE *trcfile, DC_CHAR *dataptr);
DC_VOID  print_data(FILE *trcfile, RR_INFO *ptr);
#endif


DC_VOID cx_daca_filename(DC_GLOB *globname,
                         DC_CHAR  locname[DVA_LOCNAME_LEN])
/*****************************************************************************/
/*                                                                           */
/* Description: This routine receives the global name and the proposed local */
/*              name before a Distribution request executed by DACA.  It may */
/*              modify the contents of locname before returning.             */
/*                                                                           */
/*              This routine is called from 4 points in DFS                  */
/*              - cf_delete_rp before DELETE_REQ processing starts           */
/*              - cf_fetch_rp before FETCH_REQ processing starts             */
/*              - twice in cf_store_rp before STORE_REQ processing starts,   */
/*                once for the file to be stored and once for the file to be */
/*                deleted (if specified in the request)                      */
/*                                                                           */
/* Parameters:  globname - SNA/FS name of object to be stored at the Target  */
/*              locname  - proposed local name of the object                 */
/*                                                                           */
/* Returns:     Nothing                                                      */
/*                                                                           */
/*****************************************************************************/
{
#ifdef DVA_TEST_USER_EXITS
  /***************************************************************************/
  /*                                                                         */
  /* Sample cx_daca_filename user exit                                       */
  /*                                                                         */
  /***************************************************************************/

  char * trcfile_path ;
  FILE * trcfile ;

  /***************************************************************************/
  /* Look for fnduex_trc environment variable                                */
  /***************************************************************************/
  trcfile_path = getenv(FNDUEX_TRC) ;

  /***************************************************************************/
  /* if fnduex_trc is not null, it is considered as the full path name       */
  /* of a trace file                                                         */
  /***************************************************************************/
  if ( (trcfile_path != NULL) && (*trcfile_path != '\0') )
  {
    /*************************************************************************/
    /* Open the trace file in append mode                                    */
    /*************************************************************************/
    trcfile = fopen(trcfile_path, "a" ) ;
    if (trcfile != NULL)
    {
      /***********************************************************************/
      /* Append some data                                                    */
      /***********************************************************************/
      fprintf(trcfile, "\n\nHallo, I'm the cx_daca_filename user exit!\n\n") ;
      fprintf(trcfile, "\nBye!\n\n") ;
    }
    fclose(trcfile) ;
  }
#endif
}


DC_VOID cx_daca_report(RR_INFO *report_ptr)
/*****************************************************************************/
/*                                                                           */
/* Description: This routine receives a copy of the report that DACA is      */
/*              about to send to the Server.                                 */
/*                                                                           */
/*              This routine is called from a single point in DPS, the       */
/*              send_report routine.                                         */
/*                                                                           */
/* Parameters:  report_ptr - pointer to RR_INFO structure containing a copy  */
/*                           of the report                                   */
/*                                                                           */
/* Returns:     Nothing                                                      */
/*                                                                           */
/*****************************************************************************/
{
#ifdef DVA_TEST_USER_EXITS
  /***************************************************************************/
  /*                                                                         */
  /* Sample cx_daca_report user exit                                         */
  /*                                                                         */
  /***************************************************************************/

  char * trcfile_path ;
  FILE * trcfile ;
  /***************************************************************************/
  /* Look for fnduex_trc environment variable                                */
  /***************************************************************************/
  trcfile_path = getenv(FNDUEX_TRC) ;

  /***************************************************************************/
  /* if fnduex_trc is not null, it is considered as the full path name       */
  /* of a trace file                                                         */
  /***************************************************************************/
  if ( (trcfile_path != NULL) && (*trcfile_path != '\0') )
  {
    /*************************************************************************/
    /* Open the trace file in append mode                                    */
    /*************************************************************************/
    trcfile = fopen(trcfile_path, "a" ) ;
    if (trcfile != NULL)
    {
      /***********************************************************************/
      /* Append some data                                                    */
      /***********************************************************************/
      fprintf(trcfile, "\n\nHallo, I'm the cx_daca_report user exit!\n\n") ;

      print_data(trcfile, report_ptr);

      fprintf(trcfile, "\nBye!\n\n") ;

      fclose(trcfile) ;
    }
  }
#endif
}


DC_VOID cx_daca_request(RR_INFO          *request_ptr,
                        CX_USER_RESPONSE *puserrsp)
/*****************************************************************************/
/*                                                                           */
/* Description: This routine is called by DACA on receipt of a D&CC request  */
/*              from the Server.                                             */
/*                                                                           */
/*              This routine is called from a single point in DPS, the       */
/*              request_manager routine.                                     */
/*                                                                           */
/* Parameters:  request_ptr - pointer to RR_INFO structure containing a      */
/*                            copy of the D&CC request from the server       */
/*              puserrsp    - pointer to user response structure             */
/*                                                                           */
/* Returns:     Nothing                                                      */
/*                                                                           */
/*****************************************************************************/
{
#ifdef DVA_TEST_USER_EXITS
  /***************************************************************************/
  /*                                                                         */
  /* Sample cx_daca_request user exit                                        */
  /*                                                                         */
  /***************************************************************************/

  char * trcfile_path ;
  FILE * trcfile ;
  /***************************************************************************/
  /* Look for fnduex_trc environment variable                                */
  /***************************************************************************/
  trcfile_path = getenv(FNDUEX_TRC);

  /***************************************************************************/
  /* if fnduex_trc is not null, it is considered as the full path name       */
  /* of a trace file                                                         */
  /***************************************************************************/
  if ( (trcfile_path != NULL) && (*trcfile_path != '\0') )
  {
    /*************************************************************************/
    /* Open the trace file in append mode                                    */
    /*************************************************************************/
    trcfile = fopen(trcfile_path, "a" ) ;
    if (trcfile != NULL)
    {
      /***********************************************************************/
      /* Append some data                                                    */
      /***********************************************************************/
      fprintf(trcfile, "\n\nHallo, I'm the cx_daca_request user exit!\n\n") ;

      print_data(trcfile, request_ptr);

      fprintf(trcfile, "\nBye!\n\n") ;
      fclose(trcfile) ;
    }
  }
#endif
}


DC_VOID sx_server_report(RR_INFO *report_ptr)
/*****************************************************************************/
/*                                                                           */
/* Description: This routine receives a copy of the report when the target   */
/*              has already processed the request. It can use the report     */
/*              contents to get statistical report or whatever the user can  */
/*              ask.                                                         */
/*                                                                           */
/*              This routine is called from a single point in the Scheduler  */
/*              manage_reports routine.                                      */
/*                                                                           */
/* Parameters:  report_ptr - pointer to RR_INFO structure containing a copy  */
/*                           of the report                                   */
/*                                                                           */
/* Returns:     Nothing                                                      */
/*                                                                           */
/*****************************************************************************/
{
#ifdef DVA_TEST_USER_EXITS
  /***************************************************************************/
  /*                                                                         */
  /* Sample sx_server_report user exit                                       */
  /*                                                                         */
  /***************************************************************************/

  char * trcfile_path ;
  FILE * trcfile ;
  /***************************************************************************/
  /* Look for fnduex_trc environment variable                                */
  /***************************************************************************/
  trcfile_path = getenv(FNDUEX_TRC) ;

  /***************************************************************************/
  /* if fnduex_trc is not null, it is considered as the full path name       */
  /* of a trace file                                                         */
  /***************************************************************************/
  if ( (trcfile_path != NULL) && (*trcfile_path != '\0') )
  {
    /*************************************************************************/
    /* Open the trace file in append mode                                    */
    /*************************************************************************/
    trcfile = fopen(trcfile_path, "a" ) ;
    if (trcfile != NULL)
    {
      /***********************************************************************/
      /* Append some data                                                    */
      /***********************************************************************/
      fprintf(trcfile, "\n\nHallo, I'm the sx_server_report user exit!\n\n") ;

      print_data(trcfile, report_ptr);
      fprintf(trcfile,
                   "Destination connection is:   %s\n",
                                               report_ptr->dest_conn) ;
      fprintf(trcfile,
                   "History reset is:            %d\n",
                                               report_ptr->history_reset) ;
      if (*report_ptr->dest_conn != '\0')
      {
        print_address_list(trcfile, (report_ptr->vector+4));
      }

      fprintf(trcfile, "\nBye!\n\n") ;
      fclose(trcfile) ;
    }
  }
#endif
}


DC_VOID sx_server_request(RR_INFO           *request_ptr,
                          CX_USER_RESPONSE  *puserrsp)
/*****************************************************************************/
/*                                                                           */
/* Description: This routine receives a copy of the server request when it   */
/*              scheduler is going to process it. It can use the request     */
/*              contents to get statistical report or whatever the user can  */
/*              ask.                                                         */
/*                                                                           */
/*              This routine is called from a single point in the Scheduler  */
/*              manage_requests routine.                                     */
/*                                                                           */
/* Parameters:  request_ptr - pointer to RR_INFO structure containing a      */
/*                            copy of the D&CC request from the server       */
/*              puserrsp    - pointer to user response structure             */
/*                                                                           */
/* Returns:     Nothing                                                      */
/*                                                                           */
/*****************************************************************************/
{
#ifdef DVA_TEST_USER_EXITS
  /***************************************************************************/
  /*                                                                         */
  /* Sample sx_server_request user exit                                      */
  /*                                                                         */
  /***************************************************************************/

  char * trcfile_path ;
  FILE * trcfile ;
  /***************************************************************************/
  /* Look for fnduex_trc environment variable                                */
  /***************************************************************************/
  trcfile_path = getenv(FNDUEX_TRC) ;

  /***************************************************************************/
  /* if fnduex_trc is not null, it is considered as the full path name       */
  /* of a trace file                                                         */
  /***************************************************************************/
  if ( (trcfile_path != NULL) && (*trcfile_path != '\0') )
  {
    /*************************************************************************/
    /* Open the trace file in append mode                                    */
    /*************************************************************************/
    trcfile = fopen(trcfile_path, "a" ) ;
    if (trcfile != NULL)
    {
      /***********************************************************************/
      /* Append some data                                                    */
      /***********************************************************************/
      fprintf(trcfile, "\n\nHallo, I'm the sx_server_request user exit!\n\n") ;

      print_data(trcfile, request_ptr);
      fprintf(trcfile,
                   "Destination connection is:   %s\n",
                                               request_ptr->dest_conn) ;
      fprintf(trcfile,
                   "History reset is:            %d\n",
                                           request_ptr->history_reset) ;
      if (*request_ptr->dest_conn != '\0')
      {
        print_address_list(trcfile, (request_ptr->vector+4));
      }

      fprintf(trcfile, "\nBye!\n\n") ;
      fclose(trcfile) ;
    }
  }
#endif
}


#ifdef DVA_TEST_USER_EXITS
DC_VOID  dump_dtime(FILE *trcfile, SCHED_TIME *dtime, DC_CHAR *text)
/*****************************************************************************/
/*                                                                           */
/* Description:                                                              */
/*   Print to file date and time passed in with text supplied                */
/*                                                                           */
/* Parameters:                                                               */
/*   trcfile               - pointer to the trace file                       */
/*   dtime                 - pointer to structure to dump                    */
/*   text                  - text to print with date and time info           */
/*                                                                           */
/* Returns:                                                                  */
/*   Nothing                                                                 */
/*                                                                           */
/*****************************************************************************/
{

  fprintf(trcfile, "%s%d/%d/%u  %d:%d:%d  GMT: %d\n", text,
                   dtime->date.day,
                   dtime->date.month,
                   dtime->date.year,
                   dtime->time.hour,
                   dtime->time.minute,
                   dtime->time.second,
                   dtime->time.gmt);

  return;
}

DC_VOID glob_to_dot(DC_CHAR *dname, DC_GLOB *gname)
/*****************************************************************************/
/*                                                                           */
/* Description: Concatenate a Global name in parts to a single ASCII string  */
/*              in dot format (A.B.C)                                        */
/*                                                                           */
/* Parameters:                                                               */
/*              dname - buffer to hold concatenated name in dot format       */
/*              gname - pointer to the global name                           */
/*                                                                           */
/* Returns:     Nothing                                                      */
/*                                                                           */
/*****************************************************************************/
{
   DC_USHORT  ii;                       /* Loop counter                      */

/*****************************************************************************/
/* Copy the entire global name string to the dname string (memcpy)           */
/* Loop through the token offsets (starting with token[1])                   */
/*   If this offset marks the end of the name, break                         */
/*   Copy a '.' over the NULL at (token offset - 1)                          */
/* End token loop                                                            */
/*****************************************************************************/
   memcpy(dname, gname->name, DVA_GLOB_LEN);
   if (gname->tokens[0] != DVA_END_TKN)
   {
     for (ii = 1;
          ii < DVA_NUM_GN_COMP && gname->tokens[ii] != DVA_END_TKN;
          ii++)
     {
       dname[gname->tokens[ii] - 1] = '.';
     }
   }
   return;
} /* co_glob_to_dot */

DC_VOID print_address_list(FILE *trcfile, DC_CHAR *dataptr)
/*****************************************************************************/
/*                                                                           */
/* Description: Scans the V_ADDRESS_LIST area (see fndshrr.h) and print      */
/*              in a file the content.                                       */
/*              This area is filled in every time the field dest_conn in     */
/*              RR_INFO is not empty.                                        */
/*                                                                           */
/* Parameters:                                                               */
/*              trcfile - pointer to the trace file                          */
/*              dataptr - pointer to the V_ADDRESS_LIST                      */
/*                                                                           */
/* Returns:     Nothing                                                      */
/*                                                                           */
/*****************************************************************************/
{
  V_ADDR_STRUCT  *v_addr_struct=NULL;
  V_ADDR_BLOCK   *v_addr_block=NULL;
  DC_USHORT      ii,jj;
  DC_ULONG       block_size;

  fprintf(trcfile, "\nV ADDRESS LIST is:             \n");
  v_addr_struct = (V_ADDR_STRUCT *)dataptr;
  fprintf(trcfile, "   v_addr_struct.origin          = %.8s.%.8s\n",
          v_addr_struct->origin.rgn, v_addr_struct->origin.ren);
  fprintf(trcfile, "   v_addr_struct.source          = %.8s.%.8s\n",
          v_addr_struct->source.rgn, v_addr_struct->source.ren);
  fprintf(trcfile, "   v_addr_struct.int_source      = %.8s.%.8s\n",
          v_addr_struct->int_source.rgn, v_addr_struct->int_source.ren);
  fprintf(trcfile, "   v_addr_struct.report_to       = %.8s.%.8s\n",
          v_addr_struct->report_to.rgn, v_addr_struct->report_to.ren);
  fprintf(trcfile, "   v_addr_struct.origin_server   = %.8s.%.8s\n",
          v_addr_struct->origin_server.rgn, v_addr_struct->origin_server.ren);
  fprintf(trcfile, "   v_addr_struct.block_count     = %hu\n\n",
          v_addr_struct->block_count);
  v_addr_block=(V_ADDR_BLOCK *) ((DC_CHAR *)dataptr + sizeof(V_ADDR_STRUCT));

  /**************************************************************************/
  /* display whole destination list                                         */
  /**************************************************************************/

  for (ii=0;ii<v_addr_struct->block_count ;ii++ )
  {
    fprintf(trcfile, "     v_addr_block[%hu].server = %.8s.%.8s\n",
           ii, v_addr_block->server.rgn, v_addr_block->server.ren);
    fprintf(trcfile, "     v_addr_block[%hu].trg_cnt= %hu\n",
           ii, v_addr_block->target_count);
    for (jj=0;jj<v_addr_block->target_count ;jj++ )
    {
      fprintf(trcfile, "       v_addr_block[%hu].trg[%hu] = %.8s\n",
             ii, jj, v_addr_block->targets[jj]);
    }
    block_size = sizeof(V_ADDR_BLOCK) +
                    ((v_addr_block->target_count -1) * DVA_REN_LEN);
    v_addr_block=(V_ADDR_BLOCK *)
                         ((DC_CHAR *)dataptr + sizeof(V_ADDR_STRUCT) +
                               ((ii + 1) * block_size) );
  }
  return;
}

DC_VOID print_data(FILE *trcfile, RR_INFO *ptr)
/*****************************************************************************/
/*                                                                           */
/* Description:                                                              */
/*   Print to file fields of RR_INFO structure                               */
/*                                                                           */
/* Parameters:                                                               */
/*   trcfile               - pointer to the trace file                       */
/*   ptr                   - pointer to RR_INFO area                         */
/*                                                                           */
/* Returns:                                                                  */
/*   Nothing                                                                 */
/*                                                                           */
/*****************************************************************************/
{
  char dotname[DVA_LOCNAME_LEN];
  memset(dotname, '\0', sizeof(dotname));
  fprintf(trcfile, "Originator User is:          %s\n", ptr->req_id.user);
  fprintf(trcfile, "Originator Server is:        %s\n", ptr->req_id.server);
  fprintf(trcfile, "User Sequence Number is:     %u\n",
                                                  ptr->req_id.seq_no);
  fprintf(trcfile, "Recursion Number is:         %d\n",
                                                  ptr->req_id.recursion_no);
  dump_dtime(trcfile,
             &ptr->sched_info.submit_time,
                   "Submit time is:              ");
  dump_dtime(trcfile,
                 &ptr->sched_info.sched_time,
                   "Schedule time is:            ");
  dump_dtime(trcfile,
             &ptr->sched_info.exec_time_wnd.notbef,
                   "Exec time not before is:     ");
  dump_dtime(trcfile,
             &ptr->sched_info.exec_time_wnd.notaft,
                   "Exec time not after is:      ");
  fprintf(trcfile, "Sequence Number is:          %u\n", ptr->seq_no) ;
  fprintf(trcfile, "Originator is:               %s\n", ptr->originator) ;
  fprintf(trcfile, "Destination is:              %s\n", ptr->destination) ;
  fprintf(trcfile, "Source is:                   %s\n", ptr->source) ;
  fprintf(trcfile, "User is:                     %s\n", ptr->user) ;
  fprintf(trcfile, "File name is :               %s\n", ptr->filename) ;
  glob_to_dot(dotname, &ptr->global_name);
  fprintf(trcfile, "Global name is :             %s\n", dotname) ;
  return;
}

#endif

/* fndcx.c */
